@@ -33,7 +33,7 @@ use crate::{
 /// this means `a->aa` is valid and produces a tensor with the input on the diagonal
 /// also `aa->aa` only copies the diagonal, and ignores the rest
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Einsum {
     pub args: EinsumArgs,
     pub out_axes: EinsumAxes,
@@ -272,6 +272,24 @@ impl Einsum {
         Ok(out.into_init())
     }
 
+    #[getter]
+    fn args(&self) -> PyEinsumArgs {
+        PyEinsumArgs(self.args.clone())
+    }
+
+    #[getter]
+    fn out_axes(&self) -> PyEinsumAxes {
+        PyEinsumAxes(self.out_axes.clone())
+    }
+
+    fn all_input_circuits(&self) -> Vec<CircuitRc> {
+        self.input_circuits().collect()
+    }
+
+    fn all_input_axes(&self) -> Vec<PyEinsumAxes> {
+        self.input_axes().cloned().map(PyEinsumAxes).collect()
+    }
+
     #[staticmethod]
     #[args(nodes = "*", name)]
     pub fn from_einsum_string(
@@ -294,6 +312,16 @@ impl Einsum {
         Self::try_new(zip(nodes, input_ints).collect(), output_ints, name)
     }
 
+    #[staticmethod]
+    #[args(nodes = "*", name)]
+    pub fn from_spec_py(
+        spec: EinsumSpec,
+        nodes: Vec<CircuitRc>,
+        name: Option<String>,
+    ) -> Result<Self> {
+        Self::try_from_spec(&spec, &nodes, name)
+    }
+
     #[staticmethod]
     pub fn new_diag(node: CircuitRc, ints: EinsumAxes, name: Option<String>) -> Self {
         let deduped = ints.iter().unique().cloned().collect();
@@ -307,71 +335,17 @@ impl Einsum {
     }
 
     #[staticmethod]
-    #[args(nodes = "*", name)]
-    pub fn from_spec_py(
-        spec: EinsumSpec,
-        nodes: Vec<CircuitRc>,
+    pub fn scalar_mul(
+        node: CircuitRc,
+        scalar: f64,
         name: Option<String>,
-    ) -> Result<Self> {
-        Self::try_from_spec(&spec, &nodes, name)
-    }
-
-    #[getter]
-    fn out_axes(&self) -> PyEinsumAxes {
-        PyEinsumAxes(self.out_axes.clone())
-    }
-
-    #[getter]
-    fn args(&self) -> PyEinsumArgs {
-        PyEinsumArgs(self.args.clone())
-    }
-
-    fn all_input_circuits(&self) -> Vec<CircuitRc> {
-        self.input_circuits().collect()
-    }
-
-    fn all_input_axes(&self) -> Vec<PyEinsumAxes> {
-        self.input_axes().cloned().map(PyEinsumAxes).collect()
-    }
-
-    pub fn reduced_axes(&self) -> HashSet<u8> {
-        self.input_axes()
-            .flatten()
-            .filter(|i| !self.out_axes.contains(i))
-            .copied()
-            .collect()
-    }
-
-    pub fn next_axis(&self) -> u8 {
-        self.input_axes()
-            .flatten()
-            .max()
-            .map(|x| x + 1)
-            .unwrap_or(0)
-    }
-    pub fn get_spec(&self) -> EinsumSpec {
-        let to_usize = |vals: &[u8]| -> Vec<usize> { vals.iter().map(|&x| x as usize).collect() };
-        EinsumSpec {
-            input_ints: self.input_axes().map(|x| to_usize(x)).collect(),
-            output_ints: to_usize(&self.out_axes),
-            int_sizes: dict_to_list(
-                &self
-                    .shape_map()
-                    .unwrap()
-                    .iter()
-                    .map(|(key, val)| (*key as usize, *val))
-                    .collect(),
-                None,
-            ),
-        }
-    }
-    #[staticmethod]
-    pub fn scalar_mul(node: CircuitRc, scalar: f64, name: Option<String>) -> Self {
+        scalar_name: Option<String>,
+    ) -> Self {
         let axes: EinsumAxes = (0..node.info().rank()).map(|x| x as u8).collect();
         Einsum::try_new(
             vec![
                 (node, axes.clone()),
-                (ScalarConstant::new(scalar, sv![], None).rc(), sv![]),
+                (ScalarConstant::new(scalar, sv![], scalar_name).rc(), sv![]),
             ],
             axes,
             name,
@@ -465,6 +439,39 @@ impl Einsum {
         .unwrap()
     }
 
+    pub fn reduced_axes(&self) -> HashSet<u8> {
+        self.input_axes()
+            .flatten()
+            .filter(|i| !self.out_axes.contains(i))
+            .copied()
+            .collect()
+    }
+
+    pub fn next_axis(&self) -> u8 {
+        self.input_axes()
+            .flatten()
+            .max()
+            .map(|x| x + 1)
+            .unwrap_or(0)
+    }
+
+    pub fn get_spec(&self) -> EinsumSpec {
+        let to_usize = |vals: &[u8]| -> Vec<usize> { vals.iter().map(|&x| x as usize).collect() };
+        EinsumSpec {
+            input_ints: self.input_axes().map(|x| to_usize(x)).collect(),
+            output_ints: to_usize(&self.out_axes),
+            int_sizes: dict_to_list(
+                &self
+                    .shape_map()
+                    .unwrap()
+                    .iter()
+                    .map(|(key, val)| (*key as usize, *val))
+                    .collect(),
+                None,
+            ),
+        }
+    }
+
     pub fn normalize_ints(&self) -> Einsum {
         let spec = self.get_spec();
         let spec_normalized = spec.normalize();
@@ -479,7 +486,7 @@ impl Einsum {
 
 /// Add supports broadcasting
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Add {
     #[pyo3(get)]
     pub nodes: Vec<CircuitRc>,
@@ -564,7 +571,7 @@ impl Add {
             .iter()
             .map(|(node, weight)| {
                 if use_1_weights || *weight != 1. {
-                    Einsum::scalar_mul(node.to_owned(), *weight, None).rc()
+                    Einsum::scalar_mul(node.to_owned(), *weight, None, None).rc()
                 } else {
                     node.to_owned()
                 }
@@ -678,7 +685,7 @@ impl CircuitNodeAutoName for Add {
 }
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Rearrange {
     #[pyo3(get)]
     pub node: CircuitRc,
@@ -920,7 +927,7 @@ impl Rearrange {
 /// tensor indices which are iterated together.
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Index {
     #[pyo3(get)]
     pub node: CircuitRc,
@@ -1118,7 +1125,7 @@ impl Index {
 }
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Concat {
     pub nodes: Vec<CircuitRc>,
     pub axis: usize,
@@ -1309,7 +1316,7 @@ impl Concat {
 ///
 /// right now rewrites only work with slices, maybe will support others later
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Scatter {
     #[pyo3(get)]
     pub node: CircuitRc,
@@ -1497,7 +1504,7 @@ pub fn flat_concat_back(circuits: Vec<CircuitRc>) -> Result<(Concat, Vec<Circuit
 /// Batch dims on filter aren't supported yet
 /// different padding before and after not supported yet
 #[pyclass(unsendable, extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Conv {
     #[pyo3(get)]
     pub input: CircuitRc,
