@@ -43,7 +43,7 @@ macro_rules! circuit_node_auto_leaf_impl {
 }
 
 #[pyclass(extends=PyCircuitBase, unsendable)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct ArrayConstant {
     #[pyo3(get)]
     pub value: Tensor,
@@ -220,7 +220,7 @@ impl ArrayConstant {
 }
 
 #[pyclass(extends=PyCircuitBase, unsendable)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct Symbol {
     #[pyo3(get)]
     pub uuid: Uuid,
@@ -295,7 +295,7 @@ impl Symbol {
 }
 
 #[pyclass(extends=PyCircuitBase, unsendable)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct ScalarConstant {
     #[pyo3(get)]
     pub value: f64,
