@@ -431,7 +431,11 @@ pub trait CircuitNode: CircuitNodeInit {
     where
         Self: Clone,
     {
-        self.add(Einsum::scalar_mul(other, -1.0, None).rc(), name)
+        let other_neg_name = other.name().map(|s| format!("neg {}", s));
+        self.add(
+            Einsum::scalar_mul(other, -1.0, other_neg_name, Some("neg_1".to_owned())).rc(),
+            name,
+        )
     }
     fn mul(&self, other: CircuitRc, name: Option<String>) -> Result<Einsum>
     where
@@ -570,7 +574,7 @@ pub trait CircuitNodeUnion {
 
 // not really needed to be so pedantic with ::std::...
 #[macro_export]
-macro_rules! circuit_node_eq_ord {
+macro_rules! circuit_node_eq_ord_debug {
     ($type_name:ty) => {
         rr_util::impl_eq_by_big_hash!($type_name);
 
@@ -587,6 +591,12 @@ macro_rules! circuit_node_eq_ord {
                 Some(::std::cmp::Ord::cmp(self, other))
             }
         }
+
+        impl std::fmt::Debug for $type_name {
+            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+                f.write_str(&("(\n".to_owned() + &self.compiler_repr() + ")")) // TODO: compiler repr one iter!
+            }
+        }
     };
 }
 
@@ -595,7 +605,7 @@ macro_rules! circuit_node_eq_ord {
 #[macro_export]
 macro_rules! define_circuit_union_impl {
     [$name:ident {$($x:ident),+ $(,)?}] => {
-        #[derive(::std::fmt::Debug, ::std::clone::Clone,pyo3::FromPyObject)]
+        #[derive(::std::clone::Clone,pyo3::FromPyObject)]
         pub enum $name {
             $(
                 $x($x),
@@ -655,7 +665,7 @@ macro_rules! define_circuit_union_impl {
         }
 
 
-        circuit_node_eq_ord!($name);
+        circuit_node_eq_ord_debug!($name);
 
         paste::paste! {
             $(
@@ -967,7 +977,7 @@ define_circuit_union_special!(Var {
 #[macro_export]
 macro_rules! circuit_node_extra_impl {
     ($type_name:ident) => {
-        $crate::circuit_node_eq_ord!($type_name);
+        $crate::circuit_node_eq_ord_debug!($type_name);
 
         impl $crate::CircuitNodePrivate for $type_name {
             fn info_mut(&mut self) -> &mut $crate::CachedCircuitInfo {
