@@ -380,7 +380,7 @@ impl ModuleNodeArgSpec {
 }
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct ModuleNode {
     #[pyo3(get)]
     pub nodes: Vec<CircuitRc>,
