@@ -26,7 +26,7 @@ use crate::{
 ///   `DiscreteVar`s with the same number of samples, from being sampled all
 ///   together.
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct AutoTag {
     #[pyo3(get)]
     pub node: CircuitRc,
@@ -118,7 +118,7 @@ impl AutoTag {
 }
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct DiscreteVar {
     #[pyo3(get)]
     pub values: CircuitRc,
@@ -280,7 +280,7 @@ impl DiscreteVar {
 }
 
 #[pyclass(extends=PyCircuitBase)]
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct StoredCumulantVar {
     #[pyo3(get)]
     pub cumulants: BTreeMap<usize, CircuitRc>, // using btreemap so it always iterates in order
