@@ -1,32 +1,39 @@
 use std::{
     collections::hash_map::Entry::{self, Vacant},
-    iter::zip,
+    iter::{self, zip},
     sync::Arc,
 };
 
-use anyhow::Result;
+use anyhow::{bail, Context, Error, Result};
 use bit_iter::BitIter;
 use circuit_base::{
-    computational_node::EinsumArgs, prelude::*, Add, Concat, Einsum, GeneralFunction, Index,
-    Rearrange, ScalarConstant, Scatter, Symbol,
+    computational_node::EinsumArgs, prelude::*, Add, AutoTag, CircuitType, Concat, DiscreteVar,
+    Einsum, GeneralFunction, Index, Rearrange, ScalarConstant, Scatter, Symbol,
 };
 use itertools::Itertools;
-use pyo3::prelude::*;
+use macro_rules_attribute::apply;
+use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
     filter_by_variant,
     opt_einsum::optimize_einsum_spec_cached,
     py_types::PY_CIRCUIT_ITEMS,
+    python_error_exception,
     rearrange_spec::{shape_to_op_shape, OpSize, RInts, RearrangeSpec},
     sv,
-    tensor_util::{compose, Shape, Slice, TensorAxisIndex, TensorIndex, USlice},
+    tensor_util::{
+        check_canon_idxs, compose, IndexError, IndexTensor, Shape, Slice, TensorAxisIndex,
+        TensorIndex, USlice,
+    },
     union_find::UnionFind,
     unwrap,
     util::{
-        arc_ref_clone, filter_out_idx, filter_to_idx, intersection_all, inverse_permutation,
-        is_unique, unique_to_appearance, AxisInt, BitMask128, EinsumAxes, HashBytes,
+        arc_ref_clone, cumsum, filter_out_idx, filter_to_idx, intersection_all,
+        inverse_permutation, is_unique, unique_to_appearance, AxisInt, BitMask128, EinsumAxes,
+        HashBytes,
     },
 };
 use rustc_hash::{FxHashMap as HashMap, FxHashSet as HashSet};
+use thiserror::Error;
 
 use crate::circuit_optimizer::OptimizationContext;
 
@@ -110,7 +117,9 @@ pub fn add_deduplicate(add: &Add) -> Option<Add> {
             .map(|(hash, node)| {
                 let count = duplicate_counts[&hash];
                 if count > 1 {
-                    Einsum::scalar_mul(node, count as f64, None).rc()
+                    let node_count_name = node.name().map(|s| format!("{}_mul_count", s));
+                    let count_name = node.name().is_some().then(|| "count_from_dedup".to_owned());
+                    Einsum::scalar_mul(node, count as f64, node_count_name, count_name).rc()
                 } else {
                     node
                 }
@@ -1103,6 +1112,44 @@ pub fn rearrange_fuse(node: &Rearrange) -> Option<Rearrange> {
     })
 }
 
+pub fn axis_index_split_sections_for_concat(
+    axis_index: &TensorAxisIndex,
+    sections: &Vec<usize>,
+) -> Result<Vec<Option<TensorAxisIndex>>> {
+    let starts = cumsum(sections);
+    let out = match axis_index {
+        TensorAxisIndex::Single(single) => {
+            let single = check_canon_idxs(sections.iter().sum(), &[*single])
+                .context("out of bounds when checking index for concat push down index")?[0];
+            let outer_idx = starts.iter().take_while(|start| **start <= single).count() - 1;
+            let inner_idx = single - starts[outer_idx];
+            let mut result = vec![None; sections.len()];
+            result[outer_idx] = Some(TensorAxisIndex::Single(inner_idx as i64));
+            result
+        }
+        &TensorAxisIndex::Slice(slice) => {
+            let uslice: USlice = slice.to_uslice(sections.iter().sum());
+            let mut result = vec![None; sections.len()];
+            for (i, (start, sec_len)) in zip(starts.iter(), sections.iter()).enumerate() {
+                let stop = start + sec_len;
+                if *start < uslice.stop && stop > uslice.start {
+                    result[i] = Some(TensorAxisIndex::new_plain_slice(
+                        uslice.start.saturating_sub(*start),
+                        std::cmp::min(*sec_len, uslice.stop - start),
+                    ))
+                }
+            }
+            result
+        }
+        TensorAxisIndex::Tensor(tensor) => {
+            bail!(PushDownIndexError::ConcatSplitSectionsTensorUnsupported {
+                tensor: tensor.clone()
+            })
+        }
+    };
+    Ok(out)
+}
+
 // #[pyfunction]
 // pub fn push_down_index_small(node:&Index)->Option<CircuitRc>{
 //         // if you don't specify which_axes, choose all axes that reduce dimension
@@ -1122,10 +1169,35 @@ pub fn rearrange_fuse(node: &Rearrange) -> Option<Rearrange> {
 //             }).collect()
 //     });
 // }
-
-#[pyfunction]
-pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
-    match &**node.node {
+//
+pub fn push_down_index_raw(
+    node: &Index,
+    allow_partial_pushdown: bool,
+    mut call_on_sub: impl FnMut(Index) -> Result<CircuitRc>,
+    suffix: Option<String>,
+) -> Result<CircuitRc> {
+    let mut on_sub =
+        |node: CircuitRc, index, force_name: Option<Option<String>>| -> Result<CircuitRc> {
+            let new_name = force_name.unwrap_or_else(|| {
+                suffix
+                    .as_ref()
+                    .map(|s| node.name().map(|n_s| format!("{n_s}{s}")))
+                    .flatten()
+            });
+            call_on_sub(Index::new(node, index, new_name))
+        };
+    let out_name = node.name_cloned();
+    let partial_name = || {
+        suffix
+            .as_ref()
+            .map(|s| {
+                node.node
+                    .name()
+                    .map(|inner_name| format!("{inner_name}_partial_{s}"))
+            })
+            .flatten()
+    };
+    let out: CircuitRc = match &**node.node {
         Circuit::Add(inner) => {
             let new_operands = inner
                 .nodes_and_rank_differences()
@@ -1144,10 +1216,10 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                             }
                         })
                         .collect();
-                    Index::nrc(operand.clone(), TensorIndex(index_here), None)
+                    on_sub(operand.clone(), TensorIndex(index_here), None)
                 })
-                .collect();
-            Some(Add::nrc(new_operands, inner.name_cloned()))
+                .collect::<Result<_>>()?;
+            Add::nrc(new_operands, out_name)
         }
         Circuit::Concat(inner) => {
             let index_non_axis = TensorIndex(
@@ -1167,11 +1239,29 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
             let sections = &inner.get_sizes_at_axis();
             let old_index_on_concat_axis = node.index.0[inner.axis].clone();
             let concat_axis_indices_maybe =
-                old_index_on_concat_axis.split_sections_for_concat(sections);
+                axis_index_split_sections_for_concat(&old_index_on_concat_axis, sections);
+            let concat_axis_indices_maybe = if allow_partial_pushdown {
+                if let Err(Some(PushDownIndexError::ConcatSplitSectionsTensorUnsupported {
+                    ..
+                })) = &concat_axis_indices_maybe
+                    .as_ref()
+                    .map_err(|e| e.downcast_ref())
+                {
+                    None
+                } else {
+                    Some(concat_axis_indices_maybe?)
+                }
+            } else {
+                Some(concat_axis_indices_maybe?)
+            };
             if index_non_axis.is_identity(&inner.info().shape)
                 && concat_axis_indices_maybe.is_none()
             {
-                return None;
+                bail!(PushDownIndexError::NoopOnConcat {
+                    index_node: node.clone(),
+                    index_non_axis,
+                    index: node.index.clone()
+                })
             }
             // todo: drop concat parts that get indexed out
             let new_axis = inner.axis
@@ -1183,54 +1273,56 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                 if matches!(old_index_on_concat_axis, TensorAxisIndex::Single(_)) {
                     let mut index = index_non_axis;
                     for (i, ax) in concat_indices.iter().enumerate() {
+                        // should only be once match
                         if let Some(z) = ax {
                             index.0[inner.axis] = z.clone();
-                            return Some(Index::nrc(
-                                inner.nodes[i].clone(),
-                                index,
-                                node.name_cloned(),
-                            ));
+                            return on_sub(inner.nodes[i].clone(), index, Some(out_name));
                         }
                     }
-                    None
+                    unreachable!()
                 } else {
-                    Some(Concat::nrc(
+                    Concat::nrc(
                         zip(inner.nodes.iter(), concat_indices)
                             .filter_map(|(child, idx)| {
                                 idx.map(|idx| {
                                     let mut index = index_non_axis.clone();
                                     index.0[inner.axis] = idx;
-                                    Index::nrc(child.clone(), index, node.name_cloned())
+                                    on_sub(child.clone(), index, None)
                                 })
                             })
-                            .collect(),
+                            .collect::<Result<_>>()?,
                         new_axis,
-                        inner.name_cloned(),
-                    ))
+                        out_name,
+                    )
                 }
             } else {
                 let new_concat = Concat::nrc(
                     inner
                         .nodes
                         .iter()
-                        .map(|operand| Index::nrc(operand.clone(), index_non_axis.clone(), None))
-                        .collect(),
+                        .map(|operand| on_sub(operand.clone(), index_non_axis.clone(), None))
+                        .collect::<Result<_>>()?,
                     new_axis,
-                    inner.name_cloned(),
+                    partial_name(),
                 );
                 let final_index = TensorIndex::new_single(
                     node.index.0[inner.axis].clone(),
                     new_axis,
                     new_concat.info().rank(),
                 );
-                Some(Index::nrc(new_concat, final_index, None))
+                Index::nrc(new_concat, final_index, out_name)
             }
         }
         Circuit::GeneralFunction(inner) => {
             if !inner.spec.is_batchable()
                 || inner.spec.num_non_batchable_output_dims == inner.info().rank()
             {
-                None
+                bail!(PushDownIndexError::NoopOnGeneralFunction {
+                    index_node: node.clone(),
+                    is_batchable: inner.spec.is_batchable(),
+                    num_non_batch_out: inner.spec.num_non_batchable_output_dims,
+                    inner_rank: inner.rank()
+                })
             } else {
                 let rank_to_pass = inner.info().rank() - inner.spec.num_non_batchable_output_dims;
                 let new_operands = zip(&inner.nodes, &inner.spec.input_batchability)
@@ -1244,26 +1336,45 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                                     .chain(vec![TensorAxisIndex::IDENT; ident_rank])
                                     .collect(),
                             );
-                            Index::nrc(child.clone(), index_passed, None)
+                            on_sub(child.clone(), index_passed, None)
                         } else {
-                            child.clone()
+                            Ok(child.clone())
                         }
                     })
-                    .collect();
+                    .collect::<Result<_>>()?;
                 let new_gf =
-                    GeneralFunction::nrc(new_operands, inner.spec.clone(), inner.name_cloned());
+                    GeneralFunction::nrc(new_operands, inner.spec.clone(), out_name.clone());
+
                 let passed_rank_now = rank_to_pass
                     - (0..rank_to_pass)
                         .filter(|i| matches!(node.index.0[*i], TensorAxisIndex::Single(_)))
                         .count();
-                let index_top = TensorIndex(
-                    vec![TensorAxisIndex::IDENT; passed_rank_now]
-                        .iter()
-                        .cloned()
-                        .chain(node.index.0[rank_to_pass..].iter().cloned())
-                        .collect(),
-                );
-                Some(Index::nrc(new_gf, index_top, None))
+                let top_axis_indices: Vec<_> =
+                    node.index.0[rank_to_pass..].iter().cloned().collect();
+                if top_axis_indices
+                    .iter()
+                    .zip(&new_gf.shape()[passed_rank_now..])
+                    .all(|(x, s)| x.is_identity(*s))
+                {
+                    new_gf
+                } else if allow_partial_pushdown {
+                    let index_top = TensorIndex(
+                        vec![TensorAxisIndex::IDENT; passed_rank_now]
+                            .iter()
+                            .cloned()
+                            .chain(top_axis_indices)
+                            .collect(),
+                    );
+                    Index::nrc(new_gf.rename(partial_name()), index_top, out_name)
+                } else {
+                    bail!(PushDownIndexError::GeneralFunctionSomeAxesNotPossible {
+                        index_node: node.clone(),
+                        is_batchable: inner.spec.is_batchable(),
+                        num_non_batch_out: inner.spec.num_non_batchable_output_dims,
+                        inner_rank: inner.rank(),
+                        top_axis_indices
+                    })
+                }
             }
         }
         Circuit::Einsum(inner) => {
@@ -1313,16 +1424,22 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     if !index_here.is_identity(&node.info().shape) {
                         any_operand_index_nontrivial = true;
                     }
-                    (
-                        Index::nrc(node.clone(), index_here, None),
+                    Ok((
+                        on_sub(node.clone(), index_here, None)?,
                         ints_filter_index(ints), // todo make function work with sv
-                    )
+                    ))
                 })
-                .collect();
+                .collect::<Result<_>>()?;
             if !any_operand_index_nontrivial {
-                return None;
+                bail!(PushDownIndexError::EinsumNoop {
+                    index_node: node.clone()
+                });
             }
-            let new_einsum = Einsum::nrc(new_operands, ints_filter_index(&inner.out_axes), None);
+            let new_einsum = Einsum::nrc(
+                new_operands,
+                ints_filter_index(&inner.out_axes),
+                out_name.clone(),
+            );
             let out_index = TensorIndex(
                 zip(&inner.out_axes, &node.index.0)
                     .filter_map(|(i, idx)| match int_indices.get(i).unwrap() {
@@ -1334,11 +1451,22 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     })
                     .collect(),
             );
-            Some(Index::nrc(new_einsum, out_index, None))
+            if out_index.is_identity(new_einsum.shape()) {
+                new_einsum
+            } else if allow_partial_pushdown {
+                Index::nrc(new_einsum.rename(partial_name()), out_index, out_name)
+            } else {
+                bail!(PushDownIndexError::EinsumSomeAxesNotPossible {
+                    index_node: node.clone(),
+                    out_index,
+                    out_axes: inner.out_axes.clone(),
+                    int_indices
+                })
+            }
         }
         Circuit::Rearrange(inner) => {
             // doing this in python instead of porting to rust bc it has a lot of torch ops so likely wouldn't be faster in rust
-            let python_result: Option<(TensorIndex, RearrangeSpec)> = Python::with_gil(|py| {
+            let py_out = Python::with_gil(|py| {
                 PY_CIRCUIT_ITEMS
                     .circ_compiler_util
                     .getattr(py, "index_before_rearrange_rust")
@@ -1355,12 +1483,18 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     )
                     .unwrap()
                     .extract(py)
-                    .ok()
+                    .unwrap() // TODO: this used to not be an unwrap - when can this fail!
             });
-            python_result.map(|(index, spec)| {
-                let new_index = Index::nrc(inner.node.clone(), index, node.name_cloned());
-                Rearrange::nrc_elim_identity(new_index, spec, inner.name_cloned())
-            })
+            let (index, spec): (TensorIndex, RearrangeSpec) = if let Some(x) = py_out {
+                x
+            } else {
+                bail!(PushDownIndexError::RearrangeNotPossible {
+                    index_node: node.clone(),
+                    spec: inner.spec.clone()
+                })
+            };
+            let new_index = on_sub(inner.node.clone(), index, None)?;
+            Rearrange::nrc_elim_identity(new_index, spec, out_name)
         }
         Circuit::Scatter(inner) => {
             // for now it just pushes indexes of Single(0) through axes size 1
@@ -1386,23 +1520,86 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
             })
             .collect();
             if !did_anything {
-                return None;
+                bail!(PushDownIndexError::ScatterNoop {
+                    index_node: node.clone()
+                });
+            }
+            let inner_node = on_sub(inner.node.clone(), TensorIndex(passed_index), None)?;
+
+            let new_scatter = Scatter::nrc(
+                inner_node,
+                TensorIndex(new_scatter.iter().cloned().map(USlice::into).collect()),
+                new_scatter_shape,
+                out_name.clone(),
+            );
+
+            let out_index = TensorIndex(new_outer);
+
+            if out_index.is_identity(new_scatter.shape()) {
+                new_scatter
+            } else if allow_partial_pushdown {
+                Index::nrc(new_scatter.rename(partial_name()), out_index, out_name)
+            } else {
+                bail!(PushDownIndexError::ScatterSomeAxesNotPossible {
+                    index_node: node.clone(),
+                    out_index,
+                })
             }
-            let inner_node = Index::nrc(inner.node.clone(), TensorIndex(passed_index), None);
-            let result = Some(Index::nrc(
-                Scatter::nrc(
-                    inner_node,
-                    TensorIndex(new_scatter.iter().cloned().map(USlice::into).collect()),
-                    new_scatter_shape,
-                    None,
-                ),
-                TensorIndex(new_outer),
-                None,
-            ));
-            result
         }
-        _ => None,
-    }
+        Circuit::AutoTag(tag) => AutoTag::nrc(
+            on_sub(tag.node.clone(), node.index.clone(), None)?,
+            tag.uuid,
+            out_name,
+        ),
+        Circuit::DiscreteVar(var) => DiscreteVar::nrc(
+            on_sub(
+                var.values.clone(),
+                TensorIndex(
+                    iter::once(TensorAxisIndex::IDENT) // skip sample axis
+                        .chain(node.index.0.iter().cloned())
+                        .collect(),
+                ),
+                None,
+            )?,
+            var.probs_and_group.clone(),
+            out_name,
+        ),
+        Circuit::ScalarConstant(_) => index_merge_scalar(node).unwrap(),
+        // Circuit::Cumulant(cum) => Cumulant::nrc(
+        //     cum.dims_for_self().map(...)
+        //     out_name,
+        // ),
+        Circuit::Index(_) => bail!(Error::from(PushDownIndexError::UnimplementedType {
+            index_node: node.clone(),
+            circ_type: CircuitType::Index,
+        })
+        .context("can't push down index through other index, but you can use 'index_fuse'")),
+        inner => bail!(PushDownIndexError::UnimplementedType {
+            index_node: node.clone(),
+            circ_type: inner.type_tag()
+        }),
+    };
+
+    Ok(out)
+}
+
+// This is NOT Python's push_down_index, which applies recursively
+#[pyfunction]
+pub fn push_down_index_once(
+    node: Index,
+    allow_partial_pushdown: Option<bool>,
+    suffix: Option<String>,
+) -> Result<CircuitRc> {
+    push_down_index_raw(
+        &node,
+        allow_partial_pushdown.unwrap_or(false),
+        |x| Ok(x.rc()),
+        suffix,
+    )
+}
+
+pub fn push_down_index_op(node: &Index) -> Option<CircuitRc> {
+    push_down_index_raw(node, true, |x| Ok(x.rc()), None).ok()
 }
 
 #[pyfunction]
@@ -1683,3 +1880,62 @@ pub fn einsum_permute_to_rearrange(einsum: &Einsum) -> Option<Rearrange> {
     }
     None
 }
+
+#[apply(python_error_exception)]
+#[base_error_name(PushDownIndex)]
+#[base_exception(PyValueError)]
+#[derive(Error, Debug, Clone)]
+pub enum PushDownIndexError {
+    #[error("index_node={index_node:?} index_non_axis={index_non_axis} index={index}")]
+    NoopOnConcat {
+        index_node: Index,
+        index_non_axis: TensorIndex,
+        index: TensorIndex,
+    },
+    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank}")]
+    NoopOnGeneralFunction {
+        index_node: Index,
+        is_batchable: bool,
+        num_non_batch_out: usize,
+        inner_rank: usize,
+    },
+    #[error("index_node={index_node:?} is_batchable={is_batchable} num_non_batch_out={num_non_batch_out} inner_rank={inner_rank} top_axis_indices={top_axis_indices:?}")]
+    GeneralFunctionSomeAxesNotPossible {
+        index_node: Index,
+        is_batchable: bool,
+        num_non_batch_out: usize,
+        inner_rank: usize,
+        top_axis_indices: Vec<TensorAxisIndex>,
+    },
+    // TODO: improve errors as needed
+    #[error("index_node={index_node:?}")]
+    EinsumNoop { index_node: Index },
+    #[error("index_node={index_node:?} out_index={out_index:?} out_axes={out_axes:?} int_indices={int_indices:?}")]
+    EinsumSomeAxesNotPossible {
+        index_node: Index,
+        out_index: TensorIndex,
+        out_axes: EinsumAxes,
+        int_indices: HashMap<u8, Option<TensorAxisIndex>>,
+    },
+    #[error("index_node={index_node:?} spec={spec:?}")]
+    RearrangeNotPossible {
+        index_node: Index,
+        spec: RearrangeSpec,
+    },
+    #[error("index_node={index_node:?}")]
+    ScatterNoop { index_node: Index },
+    #[error("index_node={index_node:?} out_index={out_index:?}")]
+    ScatterSomeAxesNotPossible {
+        index_node: Index,
+        out_index: TensorIndex,
+        // TODO: more
+    },
+    #[error("index_node={index_node:?} circ_type={circ_type:?}")]
+    UnimplementedType {
+        index_node: Index,
+        circ_type: CircuitType,
+    },
+
+    #[error("tensors aren't currently supported tensor={tensor:?}")]
+    ConcatSplitSectionsTensorUnsupported { tensor: IndexTensor },
+}
