@@ -23,7 +23,7 @@ use crate::{
     compiler_heuristics::deep_maybe_distribute,
     compiler_strip::{remove_autotags, strip_names},
     deep_rewrite::{
-        compiler_simp, deep_heuristic_nest_adds, deep_optimize_einsums, deep_push_down_index,
+        compiler_simp, deep_heuristic_nest_adds, deep_optimize_einsums, deep_push_down_index_raw,
     },
     scheduled_execution::{
         circuit_to_schedule, circuit_to_schedule_naive_toposort, Schedule, SchedulingError,
@@ -263,7 +263,7 @@ pub fn optimize_circuit(circuit: CircuitRc, context: &mut OptimizationContext) -
     // originally tried push_down_index in compiler_simp, but that produced worse circuits
     // for unknown reasons, maybe i'll investigate
     let circuit = timed!(
-        apply_fn_until_same(&circuit, |x| deep_push_down_index(
+        apply_fn_until_same(&circuit, |x| deep_push_down_index_raw(
             compiler_simp(x.clone(), context),
             None
         )),
