@@ -2,7 +2,6 @@ use circuit_base::{
     deep_map_op_context, deep_map_preorder_unwrap, deep_map_unwrap, prelude::*,
     visit_circuit_unwrap, Add, Concat, GeneralFunction, Index,
 };
-use itertools::Itertools;
 use num_bigint::BigUint;
 use pyo3::prelude::*;
 use rr_util::{
@@ -346,8 +345,8 @@ pub fn compiler_simp_until_same(circ: CircuitRc) -> CircuitRc {
     })
 }
 
-#[pyfunction]
-pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> CircuitRc {
+#[pyfunction] // maybe remove
+pub fn deep_push_down_index_raw(circ: CircuitRc, min_size: Option<usize>) -> CircuitRc {
     deep_map_preorder_unwrap(circ, |circ| {
         if min_size.is_none()
             || circ
@@ -358,7 +357,7 @@ pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> Circuit
             (**circ).map_or_clone(&|index: &Index| {
                 let fused = apply_fn_until_none(index, index_fuse);
                 index_elim_identity(&fused)
-                    .unwrap_or_else(|| push_down_index(&fused).unwrap_or_else(|| fused.crc()))
+                    .unwrap_or_else(|| push_down_index_op(&fused).unwrap_or_else(|| fused.crc()))
             })
         } else {
             circ
@@ -478,7 +477,7 @@ pub fn deep_pull_concat(circuit: CircuitRc, min_size: Option<usize>) -> CircuitR
     let mut cache = Default::default();
     let pulled = deep_pull_concat_messy(circuit, min_size);
     apply_fn_until_same(&pulled, |x: &CircuitRc| {
-        deep_push_down_index(compiler_simp(x.clone(), &mut cache), min_size)
+        deep_push_down_index_raw(compiler_simp(x.clone(), &mut cache), min_size)
     })
 }
 
