@@ -358,6 +358,10 @@ impl Tensor {
         self.hash.as_ref()
     }
 
+    pub fn hash_base16(&self) -> Option<String> {
+        self.hash.as_ref().map(|x| base16::encode_lower(x))
+    }
+
     pub fn flatten(&self) -> Tensor {
         Python::with_gil(|py| {
             self.tensor
