@@ -26,7 +26,7 @@ use crate::{
     py_types::{Tensor, PY_UTILS},
     pycall, python_error_exception, sv,
     tensor_db::{get_tensor_prefix, save_tensor},
-    util::{cumsum, HashBytes},
+    util::HashBytes,
 };
 pub type Shape = Sv<[usize; 6]>; // TODO: maybe this should be struct which preserves invariant that dim shapes >= 0?
 
@@ -72,6 +72,7 @@ pub enum MiscInputError {
 
     #[error("item={item}, count={count} (needed -{count} <= {item} < {count})")]
     ItemOutOfBounds { item: i64, count: usize },
+
     #[error("Children multiple dtypes {a:?} {b:?}")]
     ChildrenMultipleDtypes {
         a: Option<String>,
@@ -90,10 +91,8 @@ pub enum MiscInputError {
 #[base_exception(PyValueError)]
 #[derive(Error, Debug, Clone)]
 pub enum IndexError {
-    #[error(
-        "Only ndim of 0 or 1 is supported for index tensors and got {ndim} (tensor: {tensor:?})"
-    )]
-    TensorNDimTooLarge { ndim: usize, tensor: Tensor },
+    #[error("Only ndim of 1 is supported for index tensors and got {ndim} (tensor: {tensor:?})")]
+    TensorNDimNot1 { ndim: usize, tensor: Tensor },
 
     #[error("Index {axis:?} out of bounds, index: {index:?} shape: {shape:?}. NB: Rust circuit slices don't saturate like Python ones do.")]
     IndexOutOfBounds {
@@ -103,8 +102,12 @@ pub enum IndexError {
         axis: TensorAxisIndex,
         l: usize,
     },
+
     #[error("index rank too high: {index_rank} vs {node_rank}")]
     IndexRankTooHigh { index_rank: usize, node_rank: usize },
+
+    #[error("slice={slice:?}")]
+    UnsupportedSlice { slice: Slice },
 }
 
 pub fn check_canon_idxs(count: usize, ints: &[i64]) -> Result<Vec<usize>> {
@@ -149,6 +152,13 @@ impl Slice {
         self.stop.map(|x| Self::to_unsigned_loc(x, l)).unwrap_or(l)
     }
 
+    pub fn to_uslice(self, l: usize) -> USlice {
+        USlice {
+            start: self.start_u(l),
+            stop: self.stop_u(l),
+        }
+    }
+
     fn size(self, l: usize) -> usize {
         self.stop_u(l).saturating_sub(self.start_u(l))
     }
@@ -322,8 +332,8 @@ impl Deref for IndexTensor {
 impl TryFrom<Tensor> for IndexTensor {
     type Error = Error;
     fn try_from(tensor: Tensor) -> Result<Self> {
-        if tensor.ndim() > 1 {
-            bail!(IndexError::TensorNDimTooLarge {
+        if tensor.ndim() != 1 {
+            bail!(IndexError::TensorNDimNot1 {
                 ndim: tensor.ndim(),
                 tensor,
             });
@@ -399,50 +409,6 @@ impl TensorAxisIndex {
         false
     }
 
-    pub fn split_sections_for_concat(
-        &self,
-        sections: &Vec<usize>,
-    ) -> Option<Vec<Option<TensorAxisIndex>>> {
-        let starts = cumsum(sections);
-        match self {
-            TensorAxisIndex::Single(single) => {
-                if single < &0 {
-                    return None;
-                }
-                let outer_idx = starts
-                    .iter()
-                    .take_while(|start| **start <= *single as usize)
-                    .count()
-                    - 1;
-                let inner_idx = *single as usize - starts[outer_idx];
-                let mut result = vec![None; sections.len()];
-                result[outer_idx] = Some(TensorAxisIndex::Single(inner_idx as i64));
-                Some(result)
-            }
-            TensorAxisIndex::Slice(slice) => {
-                let uslice: Option<USlice> = (*slice).into();
-                let uslice = uslice?;
-                let mut result = vec![None; sections.len()];
-                let mut did_any = false;
-                for (i, (start, sec_len)) in zip(starts.iter(), sections.iter()).enumerate() {
-                    let stop = start + sec_len;
-                    if *start < uslice.stop && stop > uslice.start {
-                        did_any = true;
-                        result[i] = Some(TensorAxisIndex::new_plain_slice(
-                            uslice.start.saturating_sub(*start),
-                            min(*sec_len, uslice.stop - start),
-                        ))
-                    }
-                }
-                if !did_any {
-                    return None;
-                }
-                Some(result)
-            }
-            TensorAxisIndex::Tensor(_tensor) => None,
-        }
-    }
-
     pub fn new_tensor_randint_seeded(
         length: usize,
         base_length: usize,
