@@ -1,5 +1,6 @@
 use circuit_base::{expand_node::ExpandError, ConstructError, TensorEvalError};
 use circuit_rewrites::{
+    algebraic_rewrite::PushDownIndexError,
     batching::BatchError,
     scheduled_execution::{SchedulingError, SchedulingOOMError},
 };
@@ -59,4 +60,5 @@ setup_errors!(
     SampleError,
     IndexError,
     NestEinsumsError,
+    PushDownIndexError,
 );
