@@ -33,7 +33,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
             einsum_elim_identity, einsum_elim_zero, einsum_flatten_once, einsum_merge_scalars,
             einsum_nest_path, einsum_of_permute_merge, einsum_pull_removable_axes, extract_add,
             generalfunction_pull_removable_axes, index_elim_identity, index_fuse,
-            index_merge_scalar, index_split_axes, permute_of_einsum_merge, push_down_index,
+            index_merge_scalar, index_split_axes, permute_of_einsum_merge, push_down_index_once,
             rearrange_elim_identity, rearrange_fuse, rearrange_merge_scalar, remove_add_few_input,
         },
         canonicalize::{canonicalize_node_py, deep_canonicalize_py},
@@ -49,7 +49,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         deep_rewrite::{
             compiler_simp_py, compiler_simp_step_py, compiler_simp_until_same,
             deep_heuristic_nest_adds, deep_pull_concat, deep_pull_concat_messy,
-            deep_push_down_index,
+            deep_push_down_index_raw,
         },
         diag_rewrite::{add_pull_diags, einsum_push_down_trace},
         scatter_rewrite::{
@@ -164,7 +164,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(permute_of_einsum_merge, m)?)?;
     m.add_function(wrap_pyfunction!(einsum_elim_zero, m)?)?;
     m.add_function(wrap_pyfunction!(einsum_merge_scalars, m)?)?;
-    m.add_function(wrap_pyfunction!(push_down_index, m)?)?;
+    m.add_function(wrap_pyfunction!(push_down_index_once, m)?)?;
     m.add_function(wrap_pyfunction!(index_split_axes, m)?)?;
     m.add_function(wrap_pyfunction!(add_elim_zeros, m)?)?;
 
@@ -239,7 +239,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(einsum_pull_concat, m)?)?;
     m.add_function(wrap_pyfunction!(add_pull_concat, m)?)?;
     m.add_function(wrap_pyfunction!(split_to_concat, m)?)?;
-    m.add_function(wrap_pyfunction!(deep_push_down_index, m)?)?;
+    m.add_function(wrap_pyfunction!(deep_push_down_index_raw, m)?)?;
     m.add_function(wrap_pyfunction!(deep_pull_concat_messy, m)?)?;
     m.add_function(wrap_pyfunction!(deep_pull_concat, m)?)?;
     m.add_function(wrap_pyfunction!(set_named_axes_py, m)?)?;
@@ -374,6 +374,15 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
 
+    m.add_function(wrap_pyfunction!(
+        nb_operations::index_rewrites::default_index_traversal,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        nb_operations::index_rewrites::push_down_index,
+        m
+    )?)?;
+
     // create dummy object for all the type aliases we use in the stub file
     // TODO: maybe these should somehow be the actual types we use in the stub file
     // (this is kinda annoying)
