@@ -1,12 +1,10 @@
 use std::fs;
 
-use rust_circuit::{
-    circuit::{
-        circuit_optimizer::{optimize_circuit, OptimizationContext, OptimizationSettings},
-        parse_compiler_repr_bijection,
-    },
-    rrfs::get_rrfs_dir,
+use circuit_base::parsing::parse_compiler_repr_bijection;
+use circuit_rewrites::circuit_optimizer::{
+    optimize_circuit, OptimizationContext, OptimizationSettings,
 };
+use rr_util::rrfs::get_rrfs_dir;
 
 #[test]
 #[ignore]
